@@ -136,6 +136,8 @@
 #include <pthread.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #ifdef WITH_JPEG
 // for JPEG output
@@ -339,21 +341,6 @@ static void *consumer_thread(void *arg) {
 
 	int counter = 0; // each second we save a Jpeg
 
-	// set blanking flag; is not nessary we write no own blanking(HANC) for HD board ASSY 193
-	if (mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking") != NULL) {
-		// set value
-		if (strcmp(strdup(mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking")), "false")) {
-			this->blanking = 1;
-		} else if (strcmp(strdup(mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking")), "true")) {
-			this->blanking = 0;
-		} else {
-			this->blanking = 1;
-		}
-	} else {
-		// set default value without HD board, also with blanking
-		this->blanking = 1;
-	}
-
 	// set properties (path) for device files
 	if (mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "dev_video") != NULL) {
 		this->device_file_video = strdup(mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "dev_video"));
@@ -367,6 +354,48 @@ static void *consumer_thread(void *arg) {
 		}
 	}
 
+	// Set additional device file defaults
+	struct stat st;
+	int fd = stat(this->device_file_video, &st);
+	if (fd == -1) {
+		if (this->device_file_video)
+			free(this->device_file_video);
+		this->device_file_video = strdup("/dev/sdivideotx0");
+	} else {
+		close(fd);
+	}
+	if (this->device_file_audio) {
+		fd = stat(this->device_file_audio, &st);
+		if (fd == -1) {
+			if (this->device_file_audio)
+				free(this->device_file_audio);
+			this->device_file_audio = strdup("/dev/sdiaudiotx0");
+		} else {
+			close(fd);
+		}
+	} else if (strstr(this->device_file_video, "sdivideotx")) {
+		if (this->device_file_audio)
+			free(this->device_file_audio);
+		this->device_file_audio = strdup("/dev/sdiaudiotx0");
+	}
+
+	// set blanking flag; is not nessary we write no own blanking(HANC) for HD board ASSY 193
+	if (mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking")) {
+		// set value
+		if (!strcmp( mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking"), "false")) {
+			this->blanking = 0;
+		} else if (!strcmp( mlt_properties_get(MLT_CONSUMER_PROPERTIES( consumer ), "blanking"), "true")) {
+			this->blanking = 1;
+		} else {
+			this->blanking = mlt_properties_get_int(MLT_CONSUMER_PROPERTIES( consumer ), "blanking");
+		}
+	} else if (strstr(this->device_file_video, "sdivideotx")) {
+		this->blanking = 0;
+	} else {
+		// set default value without HD board, also with blanking
+		this->blanking = 1;
+	}
+
 	// Define a frame pointer
 	mlt_frame frame;
 
@@ -379,7 +408,7 @@ static void *consumer_thread(void *arg) {
 	// Tell the framework how we want our audio and video
 	int frequency = this->audio_format.sample_rate;
 	int channels = 0;
-	int samples = mlt_sample_calculator(fps, frequency, count++);
+	int samples;
 
 	// set number of audio channels, linsys vidport model 193 is limited to 8 channels (4AES frames)
 	this->audio_format.channels = 8; /* 0,2,4,6,8 */
@@ -416,6 +445,7 @@ static void *consumer_thread(void *arg) {
 				mlt_frame_get_image(frame, &video_buffer, &this->pix_fmt, &this->width, &this->height, 1);
 
 				// Get the audio from this frame and save it to our audio_buffer
+				samples = mlt_sample_calculator(fps, frequency, count++);
 				mlt_frame_get_audio(frame, (void**) &audio_buffer_tmp, &this->audio_format.aformat, &frequency, &channels, &samples);
 
 				this->audio_format.sample_rate = frequency;
@@ -470,6 +500,7 @@ static void *consumer_thread(void *arg) {
 					} else if (save_jpegs > 0) {
 						counter++;
 					}
+					mlt_events_fire(MLT_CONSUMER_PROPERTIES( consumer ), "consumer-frame-show", frame, NULL );
 				} else {
 					mlt_log_warning(MLT_CONSUMER_SERVICE(consumer), "Videobuffer was NULL, skipping playout!\n");
 				}
