@@ -38,7 +38,7 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 
 	// Get the image
 	int error = 0;
-	*format = mlt_image_yuv422;
+	*format = mlt_image_rgb24a;
 	//mlt_frame_get_image( this, image, format, width, height, 0 );
 
 	// Only process if we have no error and a valid colour space
@@ -68,16 +68,21 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 			char *name = mlt_properties_get( properties, "_unique_id" );
 			mlt_position position = mlt_properties_get_position( MLT_FRAME_PROPERTIES( this ), name );
 			mlt_properties frame_properties = MLT_FRAME_PROPERTIES( this );
+			mlt_position in = mlt_filter_get_in( filter );
+			mlt_position out = mlt_filter_get_out( filter );
 			double consumer_ar = mlt_properties_get_double( frame_properties, "consumer_aspect_ratio" );
-			mlt_properties_set_position( MLT_TRANSITION_PROPERTIES( transition ), "in", mlt_filter_get_in( filter ) );
-			mlt_properties_set_position( MLT_TRANSITION_PROPERTIES( transition ), "out", mlt_filter_get_out( filter ) );
-			mlt_producer_seek( producer, position );
+			mlt_transition_set_in_and_out( transition, in, out );
+			if ( out > 0 ) {
+				mlt_properties_set_position( MLT_PRODUCER_PROPERTIES( producer ), "length", out - in + 1 );
+				mlt_producer_set_in_and_out( producer, in, out );
+			}
+			mlt_producer_seek( producer, position - in );
 			mlt_frame_set_position( this, position );
 			mlt_properties_pass( MLT_PRODUCER_PROPERTIES( producer ), properties, "producer." );
 			mlt_properties_pass( MLT_TRANSITION_PROPERTIES( transition ), properties, "transition." );
 			mlt_service_get_frame( MLT_PRODUCER_SERVICE( producer ), &a_frame, 0 );
-			mlt_properties_set( MLT_FRAME_PROPERTIES( a_frame ), "rescale.interp", "nearest" );
-			mlt_properties_set_int( MLT_FRAME_PROPERTIES( a_frame ), "distort", 1 );
+			mlt_frame_set_position( a_frame, position );
+//			mlt_properties_set_int( MLT_FRAME_PROPERTIES( a_frame ), "distort", 1 );
 
 			// Special case - aspect_ratio = 0
 			if ( mlt_properties_get_double( frame_properties, "aspect_ratio" ) == 0 )
@@ -89,7 +94,7 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 			mlt_transition_process( transition, a_frame, this );
 			mlt_frame_get_image( a_frame, image, format, width, height, writable );
 			mlt_properties_set_data( frame_properties, "affine_frame", a_frame, 0, (mlt_destructor)mlt_frame_close, NULL );
-			mlt_properties_set_data( frame_properties, "image", *image, *width * *height * 2, NULL, NULL );
+			mlt_properties_set_data( frame_properties, "image", *image, *width * *height * 4, NULL, NULL );
 			mlt_properties_set_data( frame_properties, "alpha", mlt_frame_get_alpha_mask( a_frame ), *width * *height, NULL, NULL );
 		}
 	}
@@ -109,7 +114,7 @@ static mlt_frame filter_process( mlt_filter this, mlt_frame frame )
 	char *name = mlt_properties_get( MLT_FILTER_PROPERTIES( this ), "_unique_id" );
 
 	// Assign the current position to the name
-	mlt_properties_set_position( properties, name, mlt_frame_get_position( frame ) - mlt_filter_get_in( this ) );
+	mlt_properties_set_position( properties, name, mlt_frame_get_position( frame ) );
 
 	// Push the frame filter
 	mlt_frame_push_service( frame, this );
